package com.chelvc.framework.base.context;

import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayDeque;
import java.util.Deque;
import java.util.Objects;
import java.util.Set;
import java.util.function.Function;
import javax.servlet.ServletRequestEvent;
import javax.servlet.ServletRequestListener;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.chelvc.framework.base.annotation.Versioning;
import com.chelvc.framework.base.model.Result;
import com.chelvc.framework.base.model.Session;
import com.chelvc.framework.base.model.Using;
import com.chelvc.framework.base.util.HttpUtils;
import com.chelvc.framework.common.model.Platform;
import com.chelvc.framework.common.model.Terminal;
import com.chelvc.framework.common.model.Version;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.fasterxml.jackson.core.JsonEncoding;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.ArrayUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.NativeWebRequest;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

/**
 * 会话上下文工具类
 *
 * @author Woody
 * @date 2023/1/1
 */
@Component
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class SessionContextHolder implements ServletRequestListener {
    /**
     * 身份标识请求头
     */
    public static final String HEADER_ID = "id";

    /**
     * 使用信息请求头
     */
    public static final String HEADER_USING = "using";

    /**
     * 应用范围请求头
     */
    public static final String HEADER_SCOPE = "scope";

    /**
     * 电话号码请求头
     */
    public static final String HEADER_MOBILE = "mobile";

    /**
     * 设备标识请求头
     */
    public static final String HEADER_DEVICE = "device";

    /**
     * 渠道来源请求头
     */
    public static final String HEADER_CHANNEL = "channel";

    /**
     * 应用平台请求头
     */
    public static final String HEADER_PLATFORM = "platform";

    /**
     * 终端编号请求头
     */
    public static final String HEADER_TERMINAL = "terminal";

    /**
     * 终端版本号请求头
     */
    public static final String HEADER_VERSION = "version";

    /**
     * 时间戳请求头
     */
    public static final String HEADER_TIMESTAMP = "timestamp";

    /**
     * 签名信息请求头
     */
    public static final String HEADER_SIGNATURE = "signature";

    /**
     * 设备指纹请求头
     */
    public static final String HEADER_FINGERPRINT = "fingerprint";

    /**
     * 认证信息请求头
     */
    public static final String HEADER_AUTHORIZATION = "authorization";

    /**
     * 响应结果包装请求/响应头
     */
    public static final String HEADER_RESPONSE_WRAPPING = "response-wrapping";

    /**
     * 空会话对象实例
     */
    private static final Session EMPTY_SESSION = new Session();

    /**
     * 用户会话上下文
     */
    private static final ThreadLocal<Deque<Session>> SESSION_CONTEXT = ThreadLocal.withInitial(ArrayDeque::new);

    /**
     * 获取当前请求对象
     *
     * @return Http请求对象
     */
    public static HttpServletRequest getRequest() {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        return ObjectUtils.ifNull(attributes, ServletRequestAttributes::getRequest);
    }

    /**
     * 获取当前响应对象
     *
     * @return Http响应对象
     */
    public static HttpServletResponse getResponse() {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        return ObjectUtils.ifNull(attributes, ServletRequestAttributes::getResponse);
    }

    /**
     * 获取请求参数
     *
     * @param name 参数名称
     * @return 参数值
     */
    public static String getParameter(@NonNull String name) {
        return getParameter(name, value -> value);
    }

    /**
     * 获取请求参数
     *
     * @param name    参数名称
     * @param adapter 参数值适配器
     * @return 参数值
     */
    public static <T> T getParameter(@NonNull String name, @NonNull Function<String, T> adapter) {
        return StringUtils.ifEmpty(ObjectUtils.ifNull(getRequest(), request -> request.getParameter(name)), adapter);
    }

    /**
     * 获取会话信息
     *
     * @return 会话信息
     */
    public static Session getSession() {
        return getSession(true);
    }

    /**
     * 设置会话信息
     *
     * @param session 会话信息
     */
    public static void setSession(Session session) {
        SESSION_CONTEXT.get().push(ObjectUtils.ifNull(session, EMPTY_SESSION));
    }

    /**
     * 获取会话信息
     *
     * @param required 会话是否是必须
     * @return 会话信息
     */
    public static Session getSession(boolean required) {
        Session session = SESSION_CONTEXT.get().peek();
        if (session == EMPTY_SESSION) {
            session = null;
        }
        return required ? Objects.requireNonNull(session, "Session has not been initialized") : session;
    }

    /**
     * 获取当前用户ID
     *
     * @return 用户ID
     */
    public static Long getId() {
        return ObjectUtils.ifNull(getSession(), Session::getId);
    }

    /**
     * 获取客户端身份标识
     *
     * @param request Http请求对象
     * @return 身份标识
     */
    public static Long getId(@NonNull HttpServletRequest request) {
        return StringUtils.ifEmpty(request.getHeader(HEADER_ID), Long::parseLong);
    }

    /**
     * 获取使用信息
     *
     * @return 使用信息
     */
    public static Using getUsing() {
        return ObjectUtils.ifNull(getSession(), Session::getUsing);
    }

    /**
     * 获取使用信息
     *
     * @param request Http请求对象
     * @return 使用信息
     */
    public static Using getUsing(@NonNull HttpServletRequest request) {
        return StringUtils.ifEmpty(request.getHeader(HEADER_USING), Using::valueOf);
    }

    /**
     * 获取当前会话请求地址
     *
     * @return 请求地址
     */
    public static String getHost() {
        return ObjectUtils.ifNull(getSession(), Session::getHost);
    }

    /**
     * 获取客户端主机地址
     *
     * @param request Http请求对象
     * @return 主机地址
     */
    public static String getHost(@NonNull HttpServletRequest request) {
        return StringUtils.ifEmpty(HttpUtils.getRequestAddress(request), (String) null);
    }

    /**
     * 获取应用范围
     *
     * @return 应用范围
     */
    public static String getScope() {
        return ObjectUtils.ifNull(getSession(), Session::getScope);
    }

    /**
     * 获取应用范围
     *
     * @param request Http请求对象
     * @return 应用范围
     */
    public static String getScope(@NonNull HttpServletRequest request) {
        return StringUtils.ifEmpty(request.getHeader(HEADER_SCOPE), (String) null);
    }

    /**
     * 获取当前会话电话号码
     *
     * @return 电话号码
     */
    public static String getMobile() {
        return ObjectUtils.ifNull(getSession(), Session::getMobile);
    }

    /**
     * 获取会话电话号码
     *
     * @param request Http请求对象
     * @return 电话号码
     */
    public static String getMobile(@NonNull HttpServletRequest request) {
        return StringUtils.ifEmpty(request.getHeader(HEADER_MOBILE), (String) null);
    }

    /**
     * 获取当前会话设备标识
     *
     * @return 设备标识
     */
    public static String getDevice() {
        return ObjectUtils.ifNull(getSession(), Session::getDevice);
    }

    /**
     * 获取客户端设备标识
     *
     * @param request Http请求对象
     * @return 设备标识
     */
    public static String getDevice(@NonNull HttpServletRequest request) {
        return StringUtils.ifEmpty(request.getHeader(HEADER_DEVICE), (String) null);
    }

    /**
     * 获取当前会话渠道来源
     *
     * @return 渠道来源
     */
    public static String getChannel() {
        return ObjectUtils.ifNull(getSession(), Session::getChannel);
    }

    /**
     * 获取客户端渠道来源
     *
     * @param request Http请求对象
     * @return 渠道来源
     */
    public static String getChannel(@NonNull HttpServletRequest request) {
        return StringUtils.ifEmpty(request.getHeader(HEADER_CHANNEL), (String) null);
    }

    /**
     * 获取当前会话平台信息
     *
     * @return 平台信息
     */
    public static Platform getPlatform() {
        return ObjectUtils.ifNull(getSession(), Session::getPlatform);
    }

    /**
     * 获取客户端平台标识
     *
     * @param request Http请求对象
     * @return 平台标识
     */
    public static Platform getPlatform(@NonNull HttpServletRequest request) {
        return StringUtils.ifEmpty(request.getHeader(HEADER_PLATFORM), Platform::valueOf);
    }

    /**
     * 获取当前会话终端信息
     *
     * @return 终端信息
     */
    public static Terminal getTerminal() {
        return ObjectUtils.ifNull(getSession(), Session::getTerminal);
    }

    /**
     * 获取客户端终端
     *
     * @param request Http请求对象
     * @return 终端标识
     */
    public static Terminal getTerminal(@NonNull HttpServletRequest request) {
        return StringUtils.ifEmpty(request.getHeader(HEADER_TERMINAL), Terminal::valueOf);
    }

    /**
     * 获取当前会话版本信息
     *
     * @return 版本信息
     */
    public static String getVersion() {
        return ObjectUtils.ifNull(getSession(), Session::getVersion);
    }

    /**
     * 获取客户端版本号
     *
     * @param request Http请求对象
     * @return 版本号
     */
    public static String getVersion(@NonNull HttpServletRequest request) {
        return StringUtils.ifEmpty(request.getHeader(HEADER_VERSION), (String) null);
    }

    /**
     * 获取请求时间戳
     *
     * @return 时间戳
     */
    public static Long getTimestamp() {
        return ObjectUtils.ifNull(getSession(), Session::getTimestamp);
    }

    /**
     * 获取请求时间戳
     *
     * @param request Http请求对象
     * @return 时间戳
     */
    public static Long getTimestamp(@NonNull HttpServletRequest request) {
        return StringUtils.ifEmpty(request.getHeader(HEADER_TIMESTAMP), Long::parseLong);
    }

    /**
     * 获取签名信息
     *
     * @return 签名信息
     */
    public static String getSignature() {
        return ObjectUtils.ifNull(getRequest(), SessionContextHolder::getSignature, () -> null);
    }

    /**
     * 获取签名信息
     *
     * @param request Http请求对象
     * @return 签名信息
     */
    public static String getSignature(@NonNull HttpServletRequest request) {
        return StringUtils.ifEmpty(request.getHeader(HEADER_SIGNATURE), (String) null);
    }

    /**
     * 获取设备指纹
     *
     * @return 设备指纹
     */
    public static String getFingerprint() {
        return ObjectUtils.ifNull(getSession(), Session::getFingerprint);
    }

    /**
     * 获取设备指纹
     *
     * @param request Http请求对象
     * @return 设备指纹
     */
    public static String getFingerprint(@NonNull HttpServletRequest request) {
        return StringUtils.ifEmpty(request.getHeader(HEADER_FINGERPRINT), (String) null);
    }

    /**
     * 获取认证信息
     *
     * @return 认证信息
     */
    public static String getAuthorization() {
        HttpServletRequest request = getRequest();
        return request == null ? null : getAuthorization(request);
    }

    /**
     * 获取认证信息
     *
     * @param request Http请求对象
     * @return 认证信息
     */
    public static String getAuthorization(@NonNull HttpServletRequest request) {
        return StringUtils.ifEmpty(request.getHeader(HEADER_AUTHORIZATION), (String) null);
    }

    /**
     * 判断是否是历史首次使用
     *
     * @return true/false
     */
    public static boolean isInitialUsing() {
        return getUsing() == Using.INITIAL;
    }

    /**
     * 判断当前用户是否已注册
     *
     * @return true/false
     */
    public static boolean isRegistered() {
        return StringUtils.nonEmpty(getMobile());
    }

    /**
     * 判断当前会话是否已认证
     *
     * @return true/false
     */
    public static boolean isAuthenticated() {
        return Objects.nonNull(getId());
    }

    /**
     * 判断当前平台是否是指定平台
     *
     * @param platform 指定平台
     * @return true/false
     */
    public static boolean isPlatform(Platform platform) {
        return platform != null && platform == getPlatform();
    }

    /**
     * 判断当前终端是否是指定终端
     *
     * @param terminal 指定终端
     * @return true/false
     */
    public static boolean isTerminal(Terminal terminal) {
        return terminal != null && terminal == getTerminal();
    }

    /**
     * 判断当前版本是否是指定版本
     *
     * @param terminal 指定终端
     * @param version  指定版本
     * @return true/false
     */
    public static boolean isVersion(Terminal terminal, String version) {
        if (terminal == null || StringUtils.isEmpty(version)) {
            return false;
        }
        Session session = getSession(false);
        return terminal == ObjectUtils.ifNull(session, Session::getTerminal)
                && version.equals(ObjectUtils.ifNull(session, Session::getVersion));
    }

    /**
     * 判断当前版本是否和指定版本相同
     *
     * @param versioning 版本控制注解实例
     * @return true/false
     */
    public static boolean isVersion(Versioning versioning) {
        if (versioning == null) {
            return false;
        }
        Session session = getSession(false);
        String channel = ObjectUtils.ifNull(session, Session::getChannel);
        Platform platform = ObjectUtils.ifNull(session, Session::getPlatform);
        Terminal terminal = ObjectUtils.ifNull(session, Session::getTerminal);
        if ((ArrayUtils.isEmpty(versioning.channels()) || ArrayUtils.contains(versioning.channels(), channel))
                && (ArrayUtils.isEmpty(versioning.platforms()) || ArrayUtils.contains(versioning.platforms(), platform))
                && (ArrayUtils.isEmpty(versioning.terminals())
                || ArrayUtils.contains(versioning.terminals(), terminal))) {
            String current = ObjectUtils.ifNull(session, Session::getVersion);
            if (StringUtils.nonEmpty(versioning.value())) {
                return Objects.equals(versioning.value(), current);
            }
            return (StringUtils.isEmpty(versioning.before())
                    || Version.isBefore(current, versioning.before(), versioning.contains()))
                    && (StringUtils.isEmpty(versioning.after())
                    || Version.isAfter(current, versioning.after(), versioning.contains()));
        }
        return false;
    }

    /**
     * 判断当前版本是否和指定版本相同
     *
     * @param versions 版本控制注解实例数组
     * @return true/false
     */
    public static boolean isVersion(Versioning... versions) {
        if (ObjectUtils.nonEmpty(versions)) {
            for (Versioning versioning : versions) {
                if (isVersion(versioning)) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * 判断当前版本是否在指定版本之前
     *
     * @param terminal 指定终端
     * @param version  指定版本
     * @return true/false
     */
    public static boolean isBeforeVersion(Terminal terminal, String version) {
        return isBeforeVersion(terminal, version, false);
    }

    /**
     * 判断当前版本是否在指定版本之前
     *
     * @param terminal 指定终端
     * @param version  指定版本
     * @param contains 是否包含当前版本号
     * @return true/false
     */
    public static boolean isBeforeVersion(Terminal terminal, String version, boolean contains) {
        if (terminal == null || StringUtils.isEmpty(version)) {
            return false;
        }
        Session session = getSession(false);
        return terminal == ObjectUtils.ifNull(session, Session::getTerminal)
                && Version.isBefore(ObjectUtils.ifNull(session, Session::getVersion), version, contains);
    }

    /**
     * 判断当前版本是否在指定版本之后
     *
     * @param terminal 指定终端
     * @param version  指定版本
     * @return true/false
     */
    public static boolean isAfterVersion(Terminal terminal, String version) {
        return isAfterVersion(terminal, version, false);
    }

    /**
     * 判断当前版本是否在指定版本之后
     *
     * @param terminal 指定终端
     * @param version  指定版本
     * @param contains 是否包含当前版本号
     * @return true/false
     */
    public static boolean isAfterVersion(Terminal terminal, String version, boolean contains) {
        if (terminal == null || StringUtils.isEmpty(version)) {
            return false;
        }
        Session session = getSession(false);
        return terminal == ObjectUtils.ifNull(session, Session::getTerminal)
                && Version.isAfter(ObjectUtils.ifNull(session, Session::getVersion), version, contains);
    }

    /**
     * 判断是否是测试账号
     *
     * @return true/false
     */
    public static boolean isTesting() {
        return isTesting(getMobile());
    }

    /**
     * 判断是否是测试账号
     *
     * @param account 账号标识
     * @return true/false
     */
    public static boolean isTesting(String account) {
        if (StringUtils.isEmpty(account)) {
            return false;
        }
        Set<?> accounts = ApplicationContextHolder.getSafeProperty("testing.account", Set.class);
        return ObjectUtils.nonEmpty(accounts) && accounts.contains(account);
    }

    /**
     * 初始化会话信息
     *
     * @param session 会话信息
     */
    public static void initializeSession(@NonNull Session session) {
        Deque<Session> deque = SESSION_CONTEXT.get();
        deque.poll();
        deque.push(session);
    }

    /**
     * 初始化会话主体
     *
     * @param id 主体标识
     * @return 会话信息
     */
    public static Session initializeSessionPrincipal(@NonNull Long id) {
        return initializeSessionPrincipal(id, true);
    }

    /**
     * 初始化会话主体
     *
     * @param id    主体标识
     * @param cover 是否覆盖已认证主体信息
     * @return 会话信息
     */
    public static Session initializeSessionPrincipal(@NonNull Long id, boolean cover) {
        return initializeSessionPrincipal(id, null, null, cover);
    }

    /**
     * 初始化会话主体
     *
     * @param id     主体标识
     * @param scope  应用范围
     * @param mobile 电话号码
     * @return 会话信息
     */
    public static Session initializeSessionPrincipal(@NonNull Long id, String scope, String mobile) {
        return initializeSessionPrincipal(id, scope, mobile, true);
    }

    /**
     * 初始化会话主体
     *
     * @param id     主体标识
     * @param scope  应用范围
     * @param mobile 电话号码
     * @param cover  是否覆盖已认证主体信息
     * @return 会话信息
     */
    public static Session initializeSessionPrincipal(@NonNull Long id, String scope, String mobile, boolean cover) {
        Deque<Session> deque = SESSION_CONTEXT.get();
        Session session = deque.peek();
        if (cover || ObjectUtils.ifNull(session, Session::getId) == null) {
            session = Session.builder().id(id).scope(scope).mobile(mobile)
                    .using(ObjectUtils.ifNull(session, Session::getUsing))
                    .host(ObjectUtils.ifNull(session, Session::getHost))
                    .device(ObjectUtils.ifNull(session, Session::getDevice))
                    .channel(ObjectUtils.ifNull(session, Session::getChannel))
                    .platform(ObjectUtils.ifNull(session, Session::getPlatform))
                    .terminal(ObjectUtils.ifNull(session, Session::getTerminal))
                    .version(ObjectUtils.ifNull(session, Session::getVersion))
                    .fingerprint(ObjectUtils.ifNull(session, Session::getFingerprint))
                    .timestamp(ObjectUtils.ifNull(session, Session::getTimestamp)).build();
            deque.poll();
            deque.push(session);
        }
        return session;
    }

    /**
     * 判断是否包含返回原始结果请求头
     *
     * @param request Http请求对象
     */
    public static boolean isResponseWrappingHeader(@NonNull NativeWebRequest request) {
        String header = request.getHeader(HEADER_RESPONSE_WRAPPING);
        return StringUtils.isEmpty(header) || Objects.equals(header, StringUtils.TRUE);
    }

    /**
     * 判断是否包含返回原始结果请求头
     *
     * @param request Http请求对象
     */
    public static boolean isResponseWrappingHeader(@NonNull HttpServletRequest request) {
        String header = request.getHeader(HEADER_RESPONSE_WRAPPING);
        return StringUtils.isEmpty(header) || Objects.equals(header, StringUtils.TRUE);
    }

    /**
     * 初始化结果包装响应头
     *
     * @param response Http响应对象
     */
    public static void initializeResponseWrappingHeader(@NonNull HttpServletResponse response) {
        initializeResponseWrappingHeader(response, true);
    }

    /**
     * 初始化结果包装响应头
     *
     * @param response Http响应对象
     * @param wrapping 是否包装请求结果
     */
    public static void initializeResponseWrappingHeader(@NonNull HttpServletResponse response, boolean wrapping) {
        response.setHeader(HEADER_RESPONSE_WRAPPING, wrapping ? StringUtils.TRUE : StringUtils.FALSE);
    }

    /**
     * 相应请求结果
     *
     * @param response Http响应对象
     * @param result   请求结果
     * @throws IOException I/O异常
     */
    public static void response(@NonNull HttpServletResponse response, @NonNull Result<?> result) throws IOException {
        response.setStatus(result.getCode());
        response.setContentType(MediaType.APPLICATION_JSON_VALUE);
        response.setCharacterEncoding(JsonEncoding.UTF8.getJavaName());
        initializeResponseWrappingHeader(response);
        OutputStream output = response.getOutputStream();
        JacksonContextHolder.write(output, result);
        output.flush();
    }

    /**
     * 获取日志信息
     *
     * @param uri     资源地址
     * @param method  请求方式
     * @param type    消息类型
     * @param message 消息内容
     * @return 消息内容
     */
    public static String getLoggingMessage(String uri, String method, Object type, String message) {
        Session session = getSession(false);
        Long id = ObjectUtils.ifNull(session, Session::getId);
        String host = ObjectUtils.ifNull(session, Session::getHost);
        String device = ObjectUtils.ifNull(session, Session::getDevice);
        Platform platform = ObjectUtils.ifNull(session, Session::getPlatform);
        Terminal terminal = ObjectUtils.ifNull(session, Session::getTerminal);
        String version = ObjectUtils.ifNull(session, Session::getVersion);
        Long timestamp = ObjectUtils.ifNull(session, Session::getTimestamp);
        return String.format("[%s] [%s] [%s] [%s] [%s] [%d] [%s %s] [%d] [%s]: %s",
                host, platform, terminal, version, device, id, method, uri, timestamp, type, message
        );
    }

    /**
     * 获取日志信息
     *
     * @param request Http请求对象
     * @param type    消息类型
     * @param message 消息内容
     * @return 消息内容
     */
    public static String getLoggingMessage(HttpServletRequest request, Object type, String message) {
        String uri = ObjectUtils.ifNull(request, HttpServletRequest::getRequestURI);
        String method = ObjectUtils.ifNull(request, HttpServletRequest::getMethod);
        return getLoggingMessage(uri, method, type, message);
    }

    /**
     * 获取日志信息
     *
     * @param request   Http请求对象
     * @param type      消息类型
     * @param exception 异常对象实例
     * @return 消息内容
     */
    public static String getLoggingMessage(HttpServletRequest request, Object type, Exception exception) {
        return getLoggingMessage(request, type, ObjectUtils.ifNull(exception, Exception::getMessage));
    }

    /**
     * 获取日志信息
     *
     * @param request   Http请求对象
     * @param status    Http状态码
     * @param exception 异常信息
     * @return 消息内容
     */
    public static String getLoggingMessage(HttpServletRequest request, HttpStatus status, String exception) {
        return getLoggingMessage(request, ObjectUtils.ifNull(status, HttpStatus::value), exception);
    }

    /**
     * 获取日志信息
     *
     * @param request   Http请求对象
     * @param status    Http状态码
     * @param exception 异常对象
     * @return 消息内容
     */
    public static String getLoggingMessage(HttpServletRequest request, HttpStatus status, Exception exception) {
        return getLoggingMessage(request, status, ObjectUtils.ifNull(exception, Exception::getMessage));
    }

    /**
     * 清空上下文信息
     */
    public static void clearSessionContext() {
        Deque<Session> sessions = SESSION_CONTEXT.get();
        sessions.poll();
        if (sessions.isEmpty()) {
            SESSION_CONTEXT.remove();
        }
    }

    /**
     * 初始化会话信息
     *
     * @param request Http请求对象
     * @return 会话信息
     */
    protected Session initializeSession(@NonNull HttpServletRequest request) {
        return Session.builder().id(getId(request)).scope(getScope(request)).using(getUsing(request))
                .host(getHost(request)).mobile(getMobile(request)).device(getDevice(request))
                .channel(getChannel(request)).platform(getPlatform(request)).terminal(getTerminal(request))
                .version(getVersion(request)).fingerprint(getFingerprint(request))
                .timestamp(getTimestamp(request)).build();
    }

    @Override
    public void requestInitialized(ServletRequestEvent event) {
        setSession(this.initializeSession((HttpServletRequest) event.getServletRequest()));
    }

    @Override
    public void requestDestroyed(ServletRequestEvent event) {
        clearSessionContext();
    }
}
