package com.chelvc.framework.base.jackson;

import java.io.IOException;

import com.chelvc.framework.base.annotation.Enumerate;
import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.model.Enumeration;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.BeanProperty;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.ser.ContextualSerializer;
import com.fasterxml.jackson.databind.ser.std.StdSerializer;
import com.google.common.collect.ImmutableMap;

/**
 * Jackson枚举对象结构化处理器
 *
 * @author Woody
 * @date 2023/1/1
 */
public class EnumerationFormatSerializer extends StdSerializer<Object> implements ContextualSerializer {
    private Enumerate annotation;
    private boolean enabled = true;

    public EnumerationFormatSerializer() {
        super(Object.class);
    }

    /**
     * 获取编号字段名称
     *
     * @return 编号字段名称
     */
    private String getCodeName() {
        String name = ObjectUtils.ifNull(this.annotation, Enumerate::codeName);
        return StringUtils.ifEmpty(name, Enumerate.DEFAULT_CODE_NAME);
    }

    /**
     * 获取描述字段名称
     *
     * @return 描述字段名称
     */
    private String getDescriptionName() {
        String name = ObjectUtils.ifNull(this.annotation, Enumerate::descriptionName);
        return StringUtils.ifEmpty(name, Enumerate.DEFAULT_DESCRIPTION_NAME);
    }

    /**
     * 获取枚举编号
     *
     * @param object 枚举对象实例
     * @return 枚举编号
     */
    private String getCode(Object object) {
        if (object instanceof Enumeration) {
            Enumeration enumeration = (Enumeration) object;
            return StringUtils.ifEmpty(StringUtils.ifEmpty(enumeration.getCode(), enumeration::name), object::toString);
        }
        return ((Enum<?>) object).name();
    }

    /**
     * 获取枚举描述
     *
     * @param object 枚举对象实例
     * @return 枚举描述
     */
    private String getDescription(Object object) {
        String description = null;
        if (object instanceof Enumeration) {
            description = ((Enumeration) object).getDescription();
        }
        if (StringUtils.nonEmpty(description)) {
            return description;
        }

        // 如果描述信息为空则从国际化文件中获取
        String code = this.getCode(object);
        return ApplicationContextHolder.getMessage(object.getClass().getName() + "." + code, code);
    }

    /**
     * 判断是否受版本控制
     *
     * @param annotation 枚举式化注解实例
     * @return true/false
     */
    private boolean isVersioning(Enumerate annotation) {
        return ObjectUtils.isEmpty(annotation.versioning()) || SessionContextHolder.isVersion(annotation.versioning());
    }

    /**
     * 判断是否是枚举属性
     *
     * @param property 属性对象
     * @return true/false
     */
    private boolean isEnumerationProperty(BeanProperty property) {
        return Enum.class.isAssignableFrom(property.getType().getRawClass())
                || Enumeration.class.isAssignableFrom(property.getType().getRawClass());
    }

    /**
     * 获取属性枚举注解实例
     *
     * @param property 属性对象
     * @return 枚举注解实例
     */
    private Enumerate getEnumerateAnnotation(BeanProperty property) {
        Enumerate annotation = property.getAnnotation(Enumerate.class);
        if (annotation == null) {
            return property.getType().getRawClass().getAnnotation(Enumerate.class);
        }
        return annotation;
    }

    @Override
    public void serialize(Object object, JsonGenerator generator, SerializerProvider provider)
            throws IOException {
        if (!this.enabled || object == null) {
            generator.writeObject(object);
        } else {
            generator.writeObject(ImmutableMap.of(
                    this.getCodeName(), this.getCode(object), this.getDescriptionName(), this.getDescription(object)
            ));
        }
    }

    @Override
    public JsonSerializer<?> createContextual(SerializerProvider provider, BeanProperty property) {
        if (property == null) {
            return provider.getDefaultNullValueSerializer();
        }
        if (!this.isEnumerationProperty(property)) {
            this.enabled = false;
        } else {
            this.annotation = this.getEnumerateAnnotation(property);
            this.enabled = this.annotation == null || this.isVersioning(this.annotation);
        }
        return this;
    }
}
