package com.chelvc.framework.base.util;

import java.util.Collection;
import java.util.Map;
import java.util.Objects;

import com.chelvc.framework.base.exception.ParameterInvalidException;
import com.chelvc.framework.base.exception.ResourceUnavailableException;
import com.chelvc.framework.base.exception.ResourceUndefinedException;
import com.chelvc.framework.common.function.Getter;
import com.chelvc.framework.common.util.ObjectUtils;
import lombok.NonNull;

/**
 * 资源工具类
 *
 * @author Woody
 * @date 2023/1/1
 */
public final class ResourceUtils {
    private ResourceUtils() {
    }

    /**
     * 资源非空校验
     *
     * @param resource 资源对象
     * @param <T>      对象类型
     * @return 非空资源
     */
    public static <T> T required(T resource) {
        return required(resource, "Resource undefined");
    }

    /**
     * 资源非空校验
     *
     * @param resource 资源对象
     * @param message  异常信息
     * @param <T>      对象类型
     * @return 非空资源
     */
    public static <T> T required(T resource, String message) {
        return required(resource, message, null);
    }

    /**
     * 资源非空校验
     *
     * @param resource 资源对象
     * @param message  异常信息
     * @param business 业务编码
     * @param <T>      对象类型
     * @return 非空资源
     */
    public static <T> T required(T resource, String message, String business) {
        required(Objects.nonNull(resource), message, business);
        return resource;
    }

    /**
     * 资源非空校验
     *
     * @param resources 资源对象数组
     * @param message   异常信息
     * @param <T>       对象类型
     * @return 非空资源
     */
    public static <T> T[] required(T[] resources, String message) {
        return required(resources, message, null);
    }

    /**
     * 资源非空校验
     *
     * @param resources 资源对象数组
     * @param message   异常信息
     * @param business  业务编码
     * @param <T>       对象类型
     * @return 非空资源
     */
    public static <T> T[] required(T[] resources, String message, String business) {
        required(ObjectUtils.nonEmpty(resources), message, business);
        return resources;
    }

    /**
     * 资源非空校验
     *
     * @param resources 资源对象字典
     * @param message   异常信息
     * @param <K>       字典键类型
     * @param <V>       字典值类型
     * @param <M>       字典类型
     * @return 非空资源
     */
    public static <K, V, M extends Map<K, V>> M required(M resources, String message) {
        return required(resources, message, null);
    }

    /**
     * 资源非空校验
     *
     * @param resources 资源对象字典
     * @param message   异常信息
     * @param business  业务编码
     * @param <K>       字典键类型
     * @param <V>       字典值类型
     * @param <M>       字典类型
     * @return 非空资源
     */
    public static <K, V, M extends Map<K, V>> M required(M resources, String message, String business) {
        required(ObjectUtils.nonEmpty(resources), message, business);
        return resources;
    }

    /**
     * 资源非空校验
     *
     * @param resources 资源对象集合
     * @param message   异常信息
     * @param <T>       对象类型
     * @return 非空资源
     */
    public static <T, C extends Collection<T>> C required(C resources, String message) {
        return required(resources, message, null);
    }

    /**
     * 资源非空校验
     *
     * @param resources 资源对象集合
     * @param message   异常信息
     * @param business  业务编码
     * @param <T>       对象类型
     * @return 非空资源
     */
    public static <T, C extends Collection<T>> C required(C resources, String message, String business) {
        required(ObjectUtils.nonEmpty(resources), message, business);
        return resources;
    }

    /**
     * 资源非空校验
     *
     * @param expression 验证表达式
     * @param message    异常信息
     */
    public static void required(boolean expression, String message) {
        required(expression, message, null);
    }

    /**
     * 资源非空校验
     *
     * @param expression 验证表达式
     * @param message    异常信息
     * @param business   业务编码
     */
    public static void required(boolean expression, String message, String business) {
        if (!expression) {
            throw new ResourceUndefinedException(message, business);
        }
    }

    /**
     * 资源可用性校验
     *
     * @param expression 验证表达式
     */
    public static void available(boolean expression) {
        available(expression, "Resource unavailable");
    }

    /**
     * 资源可用性校验
     *
     * @param message 异常信息
     */
    public static void available(String message) {
        available(message, null);
    }

    /**
     * 资源可用性校验
     *
     * @param expression 验证表达式
     * @param message    异常信息
     */
    public static void available(boolean expression, String message) {
        available(expression, message, null);
    }

    /**
     * 资源可用性校验
     *
     * @param message  异常信息
     * @param business 业务编码
     */
    public static void available(String message, String business) {
        throw new ResourceUnavailableException(message, business);
    }

    /**
     * 资源可用性校验
     *
     * @param expression 验证表达式
     * @param message    异常信息
     * @param business   业务编码
     */
    public static void available(boolean expression, String message, String business) {
        if (!expression) {
            available(message, business);
        }
    }

    /**
     * 参数有效性校验
     *
     * @param errors 无效参数映射表
     */
    public static void invalid(@NonNull Map<String, String> errors) {
        throw new ParameterInvalidException(errors);
    }

    /**
     * 参数有效性校验
     *
     * @param expression 验证表达式
     * @param errors     无效参数映射表
     */
    public static void invalid(boolean expression, @NonNull Map<String, String> errors) {
        if (!expression) {
            invalid(errors);
        }
    }

    /**
     * 参数有效性校验
     *
     * @param name    参数名称
     * @param message 异常信息
     */
    public static void invalid(@NonNull String name, @NonNull String message) {
        throw new ParameterInvalidException(name, message);
    }

    /**
     * 参数有效性校验
     *
     * @param expression 验证表达式
     * @param name       参数名称
     * @param message    异常信息
     */
    public static void invalid(boolean expression, @NonNull String name, @NonNull String message) {
        if (!expression) {
            invalid(name, message);
        }
    }

    /**
     * 参数有效性校验
     *
     * @param getter  参数Getter函数实例
     * @param message 异常信息
     * @param <T>     函数调用对象类型
     * @param <R>     函数调用返回类型
     */
    public static <T, R> void invalid(@NonNull Getter<T, R> getter, @NonNull String message) {
        throw new ParameterInvalidException(ObjectUtils.lookupGetterProperty(getter), message);
    }

    /**
     * 参数有效性校验
     *
     * @param expression 验证表达式
     * @param getter     参数Getter函数实例
     * @param message    异常信息
     * @param <T>        函数调用对象类型
     * @param <R>        函数调用返回类型
     */
    public static <T, R> void invalid(boolean expression, @NonNull Getter<T, R> getter, @NonNull String message) {
        if (!expression) {
            invalid(getter, message);
        }
    }
}
