package com.chelvc.framework.common.util;

import java.util.Collection;
import java.util.Map;
import java.util.Objects;

/**
 * 断言工具类
 *
 * @author Woody
 * @date 2023/1/1
 */
public final class AssertUtils {
    private AssertUtils() {
    }

    /**
     * 条件真假检测
     *
     * @param expression 真假表达式
     * @param message    异常消息
     */
    public static void check(boolean expression, String message) {
        if (!expression) {
            throw new IllegalArgumentException(message);
        }
    }

    /**
     * 对象非空校验
     *
     * @param object  非空对象
     * @param message 异常信息
     * @param <T>     对象类型
     * @return 非空对象
     */
    public static <T> T nonnull(T object, String message) {
        check(Objects.nonNull(object), message);
        return object;
    }

    /**
     * 字符串非空校验
     *
     * @param string  非空字符串
     * @param message 异常信息
     * @param <T>     对象类型
     * @return 非空字符串
     */
    public static <T extends CharSequence> T nonempty(T string, String message) {
        check(StringUtils.nonEmpty(string), message);
        return string;
    }

    /**
     * 数组非空校验
     *
     * @param array 字节数组
     * @return 非空数组
     */
    public static byte[] nonempty(byte[] array, String message) {
        check(ObjectUtils.nonEmpty(array), message);
        return array;
    }

    /**
     * 数组非空校验
     *
     * @param array 短整型数组
     * @return 非空数组
     */
    public static short[] nonempty(short[] array, String message) {
        check(ObjectUtils.nonEmpty(array), message);
        return array;
    }

    /**
     * 数组非空校验
     *
     * @param array 整型数组
     * @return 非空数组
     */
    public static int[] nonempty(int[] array, String message) {
        check(ObjectUtils.nonEmpty(array), message);
        return array;
    }

    /**
     * 数组非空校验
     *
     * @param array 长整型数组
     * @return 非空数组
     */
    public static long[] nonempty(long[] array, String message) {
        check(ObjectUtils.nonEmpty(array), message);
        return array;
    }

    /**
     * 数组非空校验
     *
     * @param array 单精度数组
     * @return 非空数组
     */
    public static float[] nonempty(float[] array, String message) {
        check(ObjectUtils.nonEmpty(array), message);
        return array;
    }

    /**
     * 数组非空校验
     *
     * @param array 双精度数组
     * @return 非空数组
     */
    public static double[] nonempty(double[] array, String message) {
        check(ObjectUtils.nonEmpty(array), message);
        return array;
    }

    /**
     * 数组非空校验
     *
     * @param array 字符数组
     * @return 非空数组
     */
    public static char[] nonempty(char[] array, String message) {
        check(ObjectUtils.nonEmpty(array), message);
        return array;
    }

    /**
     * 数组非空校验
     *
     * @param array 布尔数组
     * @return 非空数组
     */
    public static boolean[] nonempty(boolean[] array, String message) {
        check(ObjectUtils.nonEmpty(array), message);
        return array;
    }

    /**
     * 数组非空校验
     *
     * @param array   非空数组
     * @param message 异常信息
     * @param <T>     对象类型
     * @return 非空数组
     */
    public static <T> T[] nonempty(T[] array, String message) {
        check(ObjectUtils.nonEmpty(array), message);
        return array;
    }

    /**
     * 字典非空校验
     *
     * @param mapping 非空字典
     * @param message 异常信息
     * @param <K>     字典键类型
     * @param <V>     字典值类型
     * @param <M>     字典类型
     * @return 非空字典
     */
    public static <K, V, M extends Map<K, V>> M nonempty(M mapping, String message) {
        check(ObjectUtils.nonEmpty(mapping), message);
        return mapping;
    }

    /**
     * 集合非空校验
     *
     * @param collection 非空集合
     * @param message    异常信息
     * @param <T>        集合元素类型
     * @param <C>        集合类型
     */
    public static <T, C extends Collection<T>> C nonempty(C collection, String message) {
        check(ObjectUtils.nonEmpty(collection), message);
        return collection;
    }
}
