package com.chelvc.framework.database.context;

import java.lang.reflect.Field;
import java.sql.Connection;
import java.sql.Statement;
import java.util.Collection;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ConcurrentHashMap;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.function.Supplier;
import javax.crypto.Cipher;

import com.baomidou.mybatisplus.core.metadata.TableInfo;
import com.baomidou.mybatisplus.core.metadata.TableInfoHelper;
import com.baomidou.mybatisplus.extension.service.IService;
import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.function.Executor;
import com.chelvc.framework.common.function.Handler;
import com.chelvc.framework.common.function.Provider;
import com.chelvc.framework.common.util.AESUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.database.annotation.Confidential;
import com.chelvc.framework.database.config.DatabaseProperties;
import com.chelvc.framework.database.entity.Creatable;
import com.chelvc.framework.database.entity.Deletable;
import com.chelvc.framework.database.entity.Transactor;
import com.chelvc.framework.database.entity.TreeEntity;
import com.chelvc.framework.database.entity.Updatable;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.mapping.SqlCommandType;
import org.mybatis.spring.SqlSessionTemplate;
import org.springframework.context.ApplicationContext;
import org.springframework.util.CollectionUtils;

/**
 * 数据库操作相关上下文工具类
 *
 * @author Woody
 * @date 2023/1/1
 */
@Slf4j
public class DatabaseContextHolder {
    /**
     * 类对象/业务操作实例映射表
     */
    private static final Map<Class<?>, IService<Object>> CLASS_SERVICE_MAPPING = new ConcurrentHashMap<>();

    /**
     * 对象机密数据字段映射表
     */
    private static final Map<Class<?>, List<Field>> CONFIDENTIAL_CLASS_FIELD_MAPPING = new ConcurrentHashMap<>();
    /**
     * 配置属性
     */
    private static DatabaseProperties PROPERTIES;

    /**
     * 事务处理器
     */
    private static Transactor TRANSACTOR;

    /**
     * 全局SqlSessionTemplate实例
     */
    private static SqlSessionTemplate SESSION_TEMPLATE;

    /**
     * 数据加密密码处理器
     */
    private static final ThreadLocal<Cipher> ENCRYPT_CIPHER =
            ThreadLocal.withInitial(() -> getCipher(Cipher.ENCRYPT_MODE));

    /**
     * 数据解密密码处理器
     */
    private static final ThreadLocal<Cipher> DECRYPT_CIPHER =
            ThreadLocal.withInitial(() -> getCipher(Cipher.DECRYPT_MODE));

    /**
     * 获取配置属性
     *
     * @return 配置属性
     */
    public static DatabaseProperties getProperties() {
        if (PROPERTIES == null) {
            synchronized (DatabaseProperties.class) {
                if (PROPERTIES == null) {
                    PROPERTIES = ApplicationContextHolder.getBean(DatabaseProperties.class);
                }
            }
        }
        return PROPERTIES;
    }

    /**
     * 获取事务处理器
     *
     * @return 事务处理器
     */
    public static Transactor getTransactor() {
        if (TRANSACTOR == null) {
            synchronized (Transactor.class) {
                if (TRANSACTOR == null) {
                    TRANSACTOR = ApplicationContextHolder.getBean(Transactor.class);
                }
            }
        }
        return TRANSACTOR;
    }

    /**
     * 获取SqlSessionTemplate
     *
     * @return SqlSessionTemplate实例
     */
    public static SqlSessionTemplate getSessionTemplate() {
        if (SESSION_TEMPLATE == null) {
            synchronized (SqlSessionTemplate.class) {
                if (SESSION_TEMPLATE == null) {
                    SESSION_TEMPLATE = ApplicationContextHolder.getBean(SqlSessionTemplate.class);
                }
            }
        }
        return SESSION_TEMPLATE;
    }

    /**
     * 根据加解密模式获取密码处理器
     *
     * @param mode 加解密模式
     * @return 密码处理器
     */
    public static Cipher getCipher(int mode) {
        DatabaseProperties.Codec codec = getProperties().getCodec();
        if (StringUtils.isEmpty(codec.getSecret()) || StringUtils.isEmpty(codec.getIv())) {
            return null;
        }
        return AESUtils.getCipher(mode, codec.getSecret(), codec.getIv());
    }

    /**
     * 编码数据
     *
     * @param original 原始数据
     * @return 数据密文
     */
    public static String encode(String original) {
        if (original == null) {
            return null;
        }
        Cipher cipher = getCipher(Cipher.ENCRYPT_MODE);
        return cipher == null ? original : AESUtils.encode(cipher, original);
    }

    /**
     * 解密数据
     *
     * @param ciphertext 数据密文
     * @return 数据明文
     */
    public static String decode(String ciphertext) {
        if (ciphertext == null) {
            return null;
        }
        Cipher cipher = getCipher(Cipher.DECRYPT_MODE);
        return cipher == null ? ciphertext : AESUtils.decode(cipher, ciphertext);
    }

    /**
     * 获取数据编密码处理器
     *
     * @param consumer 密码处理器接收函数
     */
    public static void encoder(@NonNull Consumer<Cipher> consumer) {
        encoder(consumer, false);
    }

    /**
     * 获取数据编密码处理器
     *
     * @param consumer 密码处理器接收函数
     * @param nullable 密码处理器是否可以为空
     */
    public static void encoder(@NonNull Consumer<Cipher> consumer, boolean nullable) {
        Cipher cipher = ENCRYPT_CIPHER.get();
        try {
            if (nullable || Objects.nonNull(cipher)) {
                consumer.accept(cipher);
            }
        } finally {
            ENCRYPT_CIPHER.remove();
        }
    }

    /**
     * 获取数据解码密码处理器
     *
     * @param consumer 密码处理器接收函数
     */
    public static void decoder(@NonNull Consumer<Cipher> consumer) {
        decoder(consumer, false);
    }

    /**
     * 获取数据解码密码处理器
     *
     * @param consumer 密码处理器接收函数
     * @param nullable 密码处理器是否可以为空
     */
    public static void decoder(@NonNull Consumer<Cipher> consumer, boolean nullable) {
        Cipher cipher = DECRYPT_CIPHER.get();
        try {
            if (nullable || Objects.nonNull(cipher)) {
                consumer.accept(cipher);
            }
        } finally {
            DECRYPT_CIPHER.remove();
        }
    }

    /**
     * 获取原生的数据库连接
     *
     * @return 数据库连接对象
     */
    public static Connection getConnection() {
        return getSessionTemplate().getConnection();
    }

    /**
     * 执行原生SQL
     *
     * @param sql SQL语句
     * @return true/false
     * @throws Exception 执行异常
     */
    public static boolean execute(@NonNull String sql) throws Exception {
        try (Statement statement = getConnection().createStatement()) {
            return statement.execute(sql);
        }
    }

    /**
     * 事务执行方法
     *
     * @param executor 回调方法
     */
    public static void transactional(@NonNull Executor executor) {
        getTransactor().execute(executor);
    }

    /**
     * 事务执行方法
     *
     * @param executor 回调方法
     * @param <T>      对象类型
     * @return 执行结果
     */
    public static <T> T transactional(@NonNull Supplier<T> executor) {
        return getTransactor().execute(executor);
    }

    /**
     * 事务执行方法
     *
     * @param executor 回调方法
     */
    public static void transactional(@NonNull Handler executor) throws Exception {
        getTransactor().execute(executor);
    }

    /**
     * 事务执行方法
     *
     * @param executor 回调方法
     * @param <T>      对象类型
     * @return 执行结果
     * @throws Exception 执行异常
     */
    public static <T> T transactional(@NonNull Provider<T> executor) throws Exception {
        return getTransactor().execute(executor);
    }

    /**
     * 根据表名获取表信息
     *
     * @param name 表名称
     * @return 表信息
     */
    public static TableInfo getTable(String name) {
        if (StringUtils.isEmpty(name)) {
            return null;
        }
        TableInfo table = TableInfoHelper.getTableInfo(name);
        if (table == null) {
            if (name.charAt(0) == '`' && name.charAt(name.length() - 1) == '`') {
                table = TableInfoHelper.getTableInfo(name.substring(1, name.length() - 1));
            } else {
                table = TableInfoHelper.getTableInfo(String.format("`%s`", name));
            }
        }
        return table;
    }

    /**
     * 获取对象的机密字段列表
     *
     * @param clazz 对象类型
     * @return 字段列表
     */
    private static List<Field> getConfidentialFields(@NonNull Class<?> clazz) {
        return CONFIDENTIAL_CLASS_FIELD_MAPPING.computeIfAbsent(clazz, c -> {
            List<Field> fields = new LinkedList<>();
            Confidential confidential = clazz.getAnnotation(Confidential.class);
            ObjectUtils.iterateFields(clazz, (index, field) -> {
                Confidential _confidential;
                if (field.getType() == String.class
                        && ((_confidential = field.getAnnotation(Confidential.class)) != null
                        || (_confidential = confidential) != null) && _confidential.value()) {
                    fields.add(field);
                }
            });
            return fields;
        });
    }

    /**
     * 获取指定对象对应的业务操作实例
     *
     * @param clazz 数据实体对象
     * @param <T>   数据对象类型
     * @return 业务操作实例
     */
    @SuppressWarnings("unchecked")
    public static <T> IService<T> lookupEntityService(@NonNull Class<T> clazz) {
        return (IService<T>) CLASS_SERVICE_MAPPING.computeIfAbsent(clazz, c -> {
            ApplicationContext applicationContext = ApplicationContextHolder.getApplicationContext();
            IService<Object> service = applicationContext.getBeansOfType(IService.class).values().stream()
                    .filter(s -> Objects.equals(s.getEntityClass(), clazz)).findAny().orElse(null);
            return Objects.requireNonNull(service, "Service not found for entity: " + clazz.getName());
        });
    }

    /**
     * 编码敏感数据属性
     *
     * @param entity 数据实体
     * @param <T>    数据类型
     */
    public static <T> void encodeConfidentialProperties(T entity) {
        if (entity == null) {
            return;
        }
        encoder(cipher -> updateConfidentialProperties(entity, original -> AESUtils.encode(cipher, original)));
    }

    /**
     * 编码敏感数据属性
     *
     * @param entities 数据实体集合
     * @param <T>      数据类型
     */
    public static <T> void encodeConfidentialProperties(Collection<T> entities) {
        if (ObjectUtils.isEmpty(entities)) {
            return;
        }
        encoder(cipher -> updateConfidentialProperties(entities, original -> AESUtils.encode(cipher, original)));
    }

    /**
     * 解码敏感数据属性
     *
     * @param entity 数据实体
     * @param <T>    数据类型
     */
    public static <T> void decodeConfidentialProperties(T entity) {
        if (entity == null) {
            return;
        }
        decoder(cipher -> updateConfidentialProperties(entity, ciphertext -> AESUtils.decode(cipher, ciphertext)));
    }

    /**
     * 解码敏感数据属性
     *
     * @param entities 数据实体集合
     * @param <T>      数据类型
     */
    public static <T> void decodeConfidentialProperties(Collection<T> entities) {
        if (ObjectUtils.isEmpty(entities)) {
            return;
        }
        decoder(cipher -> updateConfidentialProperties(entities, ciphertext -> AESUtils.decode(cipher, ciphertext)));
    }

    /**
     * 更新敏感属性数据
     *
     * @param entities 数据实体集合
     * @param codec    敏感数据编解码回调方法
     * @param <T>      数据类型
     */
    private static <T> void updateConfidentialProperties(Collection<T> entities,
                                                         @NonNull Function<String, String> codec) {
        if (CollectionUtils.isEmpty(entities)) {
            return;
        }
        Object object = entities.iterator().next();
        if (object == null) {
            return;
        }
        List<Field> fields = getConfidentialFields(object.getClass());
        if (CollectionUtils.isEmpty(fields)) {
            return;
        }
        entities.forEach(entity -> updateConfidentialProperties(entity, fields, codec));
    }

    /**
     * 更新敏感属性数据
     *
     * @param entity 数据实体
     * @param codec  敏感数据编解码回调方法
     * @param <T>    数据类型
     */
    private static <T> void updateConfidentialProperties(T entity, @NonNull Function<String, String> codec) {
        if (entity == null) {
            return;
        }
        updateConfidentialProperties(entity, getConfidentialFields(entity.getClass()), codec);
    }

    /**
     * 更新敏感属性数据
     *
     * @param entity 数据实体
     * @param fields 敏感字段列表
     * @param codec  敏感数据编解码回调方法
     * @param <T>    数据类型
     */
    private static <T> void updateConfidentialProperties(T entity, List<Field> fields,
                                                         @NonNull Function<String, String> codec) {
        if (entity == null || CollectionUtils.isEmpty(fields)) {
            return;
        }
        boolean debug = log.isDebugEnabled();
        for (Field field : fields) {
            String value = (String) ObjectUtils.getValue(entity, field);
            if (debug) {
                log.debug("Before codec[{}]: {}", field.getName(), value);
            }
            value = codec.apply(value);
            if (debug) {
                log.debug("After codec[{}]: {}", field.getName(), value);
            }
            ObjectUtils.setValue(entity, field, value);
        }
    }

    /**
     * 初始化树形实体信息
     *
     * @param entity    数据实体
     * @param operation 操作类型
     */
    public static void initializeTreeProperties(Object entity, @NonNull SqlCommandType operation) {
        if (!(entity instanceof TreeEntity) || operation != SqlCommandType.INSERT) {
            return;
        }
        // 标记层级字段，用于参数占位
        ((TreeEntity<?, ?>) entity).setLevel(1);
        // 标记序列号字段，用于参数占位
        ((TreeEntity<?, ?>) entity).setSequence(StringUtils.EMPTY);
    }

    /**
     * 初始化创建信息
     *
     * @param entity    数据实体
     * @param operation 操作类型
     */
    public static void initializeCreatableProperties(Object entity, @NonNull SqlCommandType operation) {
        if (!(entity instanceof Creatable)
                || (operation != SqlCommandType.INSERT && operation != SqlCommandType.UPDATE)) {
            return;
        }
        Creatable<?> creatable = (Creatable<?>) entity;
        if (creatable.getCreator() == null) {
            creatable.setCreator(SessionContextHolder.getId());
        }
        if (creatable.getCreateTime() == null) {
            creatable.setCreateTime(new Date());
        }
    }

    /**
     * 初始化更新信息
     *
     * @param entity    数据实体
     * @param operation 操作类型
     */
    public static void initializeUpdatableProperties(Object entity, @NonNull SqlCommandType operation) {
        if (!(entity instanceof Updatable)
                || (operation != SqlCommandType.INSERT && operation != SqlCommandType.UPDATE)) {
            return;
        }
        Updatable<?> updatable = (Updatable<?>) entity;
        Long operator = SessionContextHolder.getId();
        if (operator != null) {
            updatable.setUpdater(operator);
        }
        updatable.setUpdateTime(new Date());
    }

    /**
     * 初始化删除标记
     *
     * @param entity    数据实体
     * @param operation 操作类型
     */
    public static void initializeDeletableProperties(Object entity, @NonNull SqlCommandType operation) {
        if (!(entity instanceof Deletable)
                || (operation != SqlCommandType.INSERT && operation != SqlCommandType.UPDATE)) {
            return;
        }
        Deletable<?> deletable = (Deletable<?>) entity;
        if (deletable.getDeleted() == null) {
            deletable.setDeleted(false);
        }
    }

    /**
     * 初始化实体默认属性值
     *
     * @param entity    数据实体
     * @param operation 操作类型
     */
    public static void initializeEntityProperties(Object entity, @NonNull SqlCommandType operation) {
        // 更新基础信息
        initializeTreeProperties(entity, operation);
        initializeCreatableProperties(entity, operation);
        initializeUpdatableProperties(entity, operation);
        initializeDeletableProperties(entity, operation);

        // 敏感数据加密
        encodeConfidentialProperties(entity);
    }
}