package com.chelvc.framework.database.interceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.base.exception.ParameterInvalidException;
import com.chelvc.framework.base.model.Result;
import com.chelvc.framework.common.util.ErrorUtils;
import lombok.extern.slf4j.Slf4j;
import org.mybatis.spring.MyBatisSystemException;
import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestControllerAdvice;

/**
 * 数据库异常处理拦截器
 *
 * @author Woody
 * @date 2023/1/1
 */
@Slf4j
@RestControllerAdvice
@Order(Ordered.HIGHEST_PRECEDENCE)
public class DatabaseExceptionInterceptor {
    /**
     * 参数异常转换
     *
     * @param request  Http请求对象
     * @param response Http响应对象
     * @param e        异常对象
     * @return 请求结果
     */
    @ResponseStatus(HttpStatus.PRECONDITION_FAILED)
    @ExceptionHandler(ParameterInvalidException.class)
    public Result<?> exception(HttpServletRequest request, HttpServletResponse response, ParameterInvalidException e) {
        log.warn(SessionContextHolder.getLoggingMessage(request, HttpStatus.PRECONDITION_FAILED, e));
        SessionContextHolder.initializeResponseWrappingHeader(response);
        return Result.build(HttpStatus.PRECONDITION_FAILED, e.getData());
    }

    /**
     * 数据持久化异常转换
     *
     * @param request  Http请求对象
     * @param response Http响应对象
     * @param e        异常对象
     * @return 请求结果
     */
    @ExceptionHandler(MyBatisSystemException.class)
    public Result<?> exception(HttpServletRequest request, HttpServletResponse response, MyBatisSystemException e) {
        ParameterInvalidException parameterInvalidException = ErrorUtils.lookup(e, ParameterInvalidException.class);
        if (parameterInvalidException != null) {
            response.setStatus(HttpStatus.PRECONDITION_FAILED.value());
            return this.exception(request, response, parameterInvalidException);
        }
        log.error(SessionContextHolder.getLoggingMessage(request, HttpStatus.INTERNAL_SERVER_ERROR, e), e);
        response.setStatus(HttpStatus.INTERNAL_SERVER_ERROR.value());
        SessionContextHolder.initializeResponseWrappingHeader(response);
        boolean wrapping = SessionContextHolder.isResponseWrappingHeader(request);
        String message = wrapping ? HttpStatus.INTERNAL_SERVER_ERROR.getReasonPhrase() : e.getMessage();
        return Result.build(HttpStatus.INTERNAL_SERVER_ERROR, null, message);
    }
}
