package com.chelvc.framework.feign.context;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import javax.servlet.ServletRequestEvent;
import javax.servlet.ServletRequestListener;

import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.base.model.Session;
import com.chelvc.framework.base.model.Using;
import com.chelvc.framework.common.model.Platform;
import com.chelvc.framework.common.model.Terminal;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import feign.RequestTemplate;
import feign.Response;
import lombok.NonNull;
import org.springframework.stereotype.Component;

/**
 * Feign上下文工具
 *
 * @author Woody
 * @date 2023/1/1
 */
@Component
public class FeignContextHolder implements ServletRequestListener {
    /**
     * 请求头上下文
     */
    private static final ThreadLocal<Map<String, String>> HEADER_CONTEXT = ThreadLocal.withInitial(HashMap::new);

    /**
     * 获取上下文请求头键/值对
     *
     * @return 请求头键/值对
     */
    public static Map<String, String> getHeaders() {
        return Collections.unmodifiableMap(HEADER_CONTEXT.get());
    }

    /**
     * 设置上下文请求头键/值对
     *
     * @param headers 请求头键/值对
     */
    public static void setHeaders(@NonNull Map<String, String> headers) {
        Map<String, String> context = HEADER_CONTEXT.get();
        context.clear();
        context.putAll(headers);
    }

    /**
     * 清空请求头
     */
    public static void clearHeaders() {
        HEADER_CONTEXT.get().clear();
    }

    /**
     * 添加请求头
     *
     * @param key   请求头键
     * @param value 请求头值
     */
    public static void addHeader(@NonNull String key, String value) {
        HEADER_CONTEXT.get().put(key, value);
    }

    /**
     * 批量添加请求头
     *
     * @param headers 请求头键/值对
     */
    public static void addHeaders(@NonNull Map<String, String> headers) {
        if (ObjectUtils.nonEmpty(headers)) {
            HEADER_CONTEXT.get().putAll(headers);
        }
    }

    /**
     * 移除请求头
     *
     * @param key 请求头键
     */
    public static void removeHeader(@NonNull String key) {
        HEADER_CONTEXT.get().remove(key);
    }

    /**
     * 移除请求头
     *
     * @param keys 请求头键数组
     */
    public static void removeHeaders(@NonNull String... keys) {
        if (ObjectUtils.nonEmpty(keys)) {
            Map<String, String> headers = HEADER_CONTEXT.get();
            for (String key : keys) {
                headers.remove(key);
            }
        }
    }

    /**
     * 移除请求头
     *
     * @param keys 请求头键集合
     */
    public static void removeHeaders(@NonNull Collection<String> keys) {
        if (ObjectUtils.nonEmpty(keys)) {
            Map<String, String> headers = HEADER_CONTEXT.get();
            keys.forEach(headers::remove);
        }
    }

    /**
     * 初始化请求头
     *
     * @param template Feign请求模版对象实例
     */
    public static void initializeRequestHeader(@NonNull RequestTemplate template) {
        // 拷贝当前会话请求头
        Session session = SessionContextHolder.getSession(false);
        if (session != null) {
            template.header(SessionContextHolder.HEADER_ID,
                    (String) ObjectUtils.ifNull(session.getId(), String::valueOf));
            template.header(SessionContextHolder.HEADER_SCOPE, session.getScope());
            template.header(SessionContextHolder.HEADER_USING, ObjectUtils.ifNull(session.getUsing(), Using::name));
            template.header(SessionContextHolder.HEADER_MOBILE, session.getMobile());
            template.header(SessionContextHolder.HEADER_DEVICE, session.getDevice());
            template.header(SessionContextHolder.HEADER_CHANNEL, session.getChannel());
            template.header(SessionContextHolder.HEADER_PLATFORM,
                    ObjectUtils.ifNull(session.getPlatform(), Platform::name));
            template.header(SessionContextHolder.HEADER_TERMINAL,
                    ObjectUtils.ifNull(session.getTerminal(), Terminal::name));
            template.header(SessionContextHolder.HEADER_VERSION, session.getVersion());
            template.header(SessionContextHolder.HEADER_TIMESTAMP,
                    (String) ObjectUtils.ifNull(session.getTimestamp(), String::valueOf));
            template.header(SessionContextHolder.HEADER_FINGERPRINT, session.getFingerprint());
            template.header("x-real-ip", session.getHost());
        }

        // 拷贝当前上下文请求头
        Map<String, String> headers = HEADER_CONTEXT.get();
        if (ObjectUtils.nonEmpty(headers)) {
            headers.forEach(template::header);
        }

        // 设置获取原始相应对象请求头
        template.header(SessionContextHolder.HEADER_RESPONSE_WRAPPING, StringUtils.FALSE);
    }

    /**
     * 判断是否包含返回原始结果请求头
     *
     * @param response 请求响应对象
     * @return true/false
     */
    public static boolean isResponseWrappingHeader(@NonNull Response response) {
        Collection<String> values = ObjectUtils.ifNull(
                response.headers(), headers -> headers.get(SessionContextHolder.HEADER_RESPONSE_WRAPPING)
        );
        return values != null && values.stream().anyMatch(StringUtils.TRUE::equals);
    }

    /**
     * 清空上下文信息
     */
    public static void clearFeignContext() {
        HEADER_CONTEXT.remove();
    }

    @Override
    public void requestDestroyed(ServletRequestEvent event) {
        clearFeignContext();
    }
}
