package com.chelvc.framework.kafka.context;

import java.nio.charset.StandardCharsets;
import java.util.Iterator;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.base.model.Session;
import com.chelvc.framework.common.util.JacksonUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.kafka.config.KafkaProperties;
import lombok.NonNull;
import org.apache.kafka.common.header.Header;
import org.apache.kafka.common.header.Headers;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.kafka.support.SendResult;
import org.springframework.util.concurrent.ListenableFuture;

/**
 * Kafka上下文操作工具类
 *
 * @author Woody
 * @date 2023/1/1
 */
public class KafkaContextHolder {
    /**
     * 会话信息标识
     */
    private static final String SESSION = "session";

    /**
     * 配置属性
     */
    private static KafkaProperties PROPERTIES;

    /**
     * 模版实例
     */
    private static KafkaTemplate<String, Object> KAFKA_TEMPLATE;

    /**
     * 获取配置属性
     *
     * @return 配置属性实例
     */
    public static KafkaProperties getProperties() {
        if (PROPERTIES == null) {
            synchronized (KafkaProperties.class) {
                if (PROPERTIES == null) {
                    PROPERTIES = ApplicationContextHolder.getBean(KafkaProperties.class);
                }
            }
        }
        return PROPERTIES;
    }

    /**
     * 获取Kafka模版实例
     *
     * @return Kafka模版实例
     */
    @SuppressWarnings("unchecked")
    public static KafkaTemplate<String, Object> getKafkaTemplate() {
        if (KAFKA_TEMPLATE == null) {
            synchronized (KafkaTemplate.class) {
                if (KAFKA_TEMPLATE == null) {
                    KAFKA_TEMPLATE = ApplicationContextHolder.getBean(KafkaTemplate.class);
                }
            }
        }
        return KAFKA_TEMPLATE;
    }

    /**
     * 消息环境隔离
     *
     * @param original 原始消息标识
     * @return 消息标识
     */
    public static String isolate(@NonNull String original) {
        String namespace = StringUtils.ifEmpty(getProperties().getNamespace(), StringUtils.EMPTY);
        return ApplicationContextHolder.getEnvironment().resolvePlaceholders(namespace + original);
    }

    /**
     * 从消息头中获取会话信息
     *
     * @param headers 消息头对象实例
     * @return 会话实例
     */
    public static Session getSession(@NonNull Headers headers) {
        Iterator<Header> iterator = headers.headers(SESSION).iterator();
        if (iterator.hasNext()) {
            return JacksonUtils.deserialize(iterator.next().value(), Session.class);
        }
        return null;
    }

    /**
     * 初始化会话信息
     *
     * @param headers 消息头对象实例
     */
    public static void initializeSession(@NonNull Headers headers) {
        Session session = SessionContextHolder.getSession(false);
        if (session != null) {
            headers.add(SESSION, JacksonUtils.serialize(session).getBytes(StandardCharsets.UTF_8));
        }
    }

    /**
     * 发送消息
     *
     * @param topic   消息主题
     * @param payload 消息内容
     * @return 发送结果
     */
    public static ListenableFuture<SendResult<String, Object>> send(@NonNull String topic, @NonNull Object payload) {
        return getKafkaTemplate().send(topic, payload);
    }

    /**
     * 发送消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param ordering 顺序消息标识
     * @return 发送结果
     */
    public static ListenableFuture<SendResult<String, Object>> send(@NonNull String topic, @NonNull Object payload,
                                                                    String ordering) {
        return getKafkaTemplate().send(topic, ordering, payload);
    }
}
