package com.chelvc.framework.oauth.token;

import java.util.Objects;

import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.oauth.config.OauthConfigurer;
import com.chelvc.framework.oauth.context.OauthContextHolder;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.http.HttpStatus;
import org.springframework.security.oauth2.core.OAuth2AuthenticationException;
import org.springframework.security.oauth2.core.OAuth2Error;
import org.springframework.security.oauth2.core.OAuth2TokenValidatorResult;
import org.springframework.security.oauth2.jwt.Jwt;
import org.springframework.security.oauth2.server.resource.BearerTokenError;
import org.springframework.stereotype.Component;

/**
 * 基于Redis的令牌有效性验证实现
 *
 * @author Woody
 * @date 2023/1/1
 * @see OauthConfigurer
 * @see TokenActiveValidator
 */
@Slf4j
@Component
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@ConditionalOnProperty(prefix = "oauth.token", name = "recheck", havingValue = "true")
public class RedisTokenActiveValidator implements TokenActiveValidator {
    private final RedisTemplate<String, Object> redisTemplate;

    @Override
    public OAuth2TokenValidatorResult validate(@NonNull Jwt jwt) {
        // 验证Redis中是否存在该令牌，同时忽略Redis请求/操作异常（为了提高系统可用性）
        String token = null;
        String key = OauthContextHolder.buildAccessTokenKey(
                OauthContextHolder.getId(jwt), SessionContextHolder.getTerminal()
        );
        try {
            if (StringUtils.isEmpty(token = (String) this.redisTemplate.opsForValue().get(key))) {
                return OAuth2TokenValidatorResult.failure(
                        new OAuth2Error("invalid_token", "Token is expired", null)
                );
            }
        } catch (Exception e) {
            log.warn("Token validate failed: {}", e.getMessage());
        }

        // 判断令牌是否相同，如果不相同则标识令牌已经被重置（账号已在其他地方登录）
        if (StringUtils.nonEmpty(token) && !Objects.equals(token, jwt.getTokenValue())) {
            throw new OAuth2AuthenticationException(
                    new BearerTokenError("invalid_token", HttpStatus.GONE, "Token has been reset", null)
            );
        }
        return OAuth2TokenValidatorResult.success();
    }
}
