package com.chelvc.framework.rocketmq.config;

import java.util.Objects;
import java.util.concurrent.atomic.AtomicLong;

import com.chelvc.framework.rocketmq.annotation.RocketMQConsumer;
import com.chelvc.framework.rocketmq.consumer.MultipleRocketMQListenerContainer;
import com.chelvc.framework.rocketmq.consumer.RocketMQListenerContainer;
import com.chelvc.framework.rocketmq.consumer.SingleRocketMQListenerContainer;
import com.chelvc.framework.rocketmq.interceptor.RocketMQListener;
import com.chelvc.framework.rocketmq.producer.DefaultRocketMQProducerFactory;
import com.chelvc.framework.rocketmq.producer.RocketMQProducerFactory;
import lombok.RequiredArgsConstructor;
import org.apache.rocketmq.client.apis.ClientConfiguration;
import org.apache.rocketmq.client.apis.ClientServiceProvider;
import org.apache.rocketmq.client.apis.SessionCredentialsProvider;
import org.apache.rocketmq.client.apis.StaticSessionCredentialsProvider;
import org.springframework.aop.framework.AopProxyUtils;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.annotation.Bean;
import org.springframework.context.support.GenericApplicationContext;
import org.springframework.stereotype.Component;

/**
 * RocketMQ配置类
 *
 * @author Woody
 * @date 2023/1/1
 */
@Component
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class RocketMQConfigurer implements ApplicationContextAware {
    private final RocketMQProperties properties;
    private final AtomicLong counter = new AtomicLong(0);

    @Bean
    public ClientServiceProvider provider() {
        return ClientServiceProvider.loadService();
    }

    @Bean
    public ClientConfiguration configuration() {
        SessionCredentialsProvider sessionCredentialsProvider =
                new StaticSessionCredentialsProvider(this.properties.getAccessKey(), this.properties.getSecretKey());
        return ClientConfiguration.newBuilder().setEndpoints(this.properties.getEndpoints())
                .setCredentialProvider(sessionCredentialsProvider).build();
    }

    @Bean
    public RocketMQProducerFactory producerFactory() {
        return new DefaultRocketMQProducerFactory(this.provider(), this.configuration());
    }

    /**
     * 注册消息监听器容器
     *
     * @param applicationContext 应用上下文实例
     * @param listener           消息监听器实例
     * @param <T>                消息对象类型
     */
    @SuppressWarnings("unchecked")
    private <T> void registerListenerContainer(ApplicationContext applicationContext, RocketMQListener<T> listener) {
        Class<?> clazz = AopProxyUtils.ultimateTargetClass(listener);
        RocketMQConsumer annotation = Objects.requireNonNull(
                clazz.getAnnotation(RocketMQConsumer.class),
                "@RocketMQConsumer annotation is missing: " + listener.getClass().getName()
        );
        String name = String.format("%s_%s", RocketMQListenerContainer.class.getName(), this.counter.incrementAndGet());
        ((GenericApplicationContext) applicationContext).registerBean(name, RocketMQListenerContainer.class, () -> {
            boolean multiple = Math.max(annotation.quantity(), 1) > 1;
            return multiple ? new MultipleRocketMQListenerContainer<>() : new SingleRocketMQListenerContainer<>();
        });
        RocketMQListenerContainer<T> container = applicationContext.getBean(name, RocketMQListenerContainer.class);
        container.initialize(this.provider(), this.configuration(), applicationContext, listener, annotation,
                this.properties);
    }

    @Override
    @SuppressWarnings("unchecked")
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        // 注册消息监听器
        applicationContext.getBeansOfType(RocketMQListener.class).values()
                .forEach(listener -> this.registerListenerContainer(applicationContext, listener));
    }
}
