package com.chelvc.framework.rocketmq.context;

import java.lang.reflect.Type;
import java.nio.charset.StandardCharsets;
import java.time.Duration;
import java.util.Date;
import java.util.Objects;
import java.util.function.Consumer;
import java.util.function.Function;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.base.model.Session;
import com.chelvc.framework.common.model.Reference;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.JacksonUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.rocketmq.config.RocketMQProperties;
import com.chelvc.framework.rocketmq.producer.RocketMQProducerFactory;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.tuple.Pair;
import org.apache.rocketmq.client.apis.ClientException;
import org.apache.rocketmq.client.apis.ClientServiceProvider;
import org.apache.rocketmq.client.apis.message.Message;
import org.apache.rocketmq.client.apis.message.MessageBuilder;
import org.apache.rocketmq.client.apis.message.MessageId;
import org.apache.rocketmq.client.apis.message.MessageView;
import org.apache.rocketmq.client.apis.producer.Producer;
import org.apache.rocketmq.client.apis.producer.Transaction;
import org.apache.rocketmq.client.apis.producer.TransactionResolution;

/**
 * RocketMQ上下文工具类
 *
 * @author Woody
 * @date 2023/1/1
 */
@Slf4j
public class RocketMQContextHolder {
    /**
     * 消息类型标识
     */
    private static final String TYPE = "type";

    /**
     * 会话信息标识
     */
    private static final String SESSION = "session";

    /**
     * 延时时间标识
     */
    private static final String DELAYING = "delaying";

    /**
     * 单次最大延迟时间（毫秒），阿里云RocketMQ延时消息最大支持24小时
     */
    private static final int MAX_DELAY_TIME = 24 * 60 * 60 * 1000;

    /**
     * 配置属性
     */
    private static RocketMQProperties PROPERTIES;

    /**
     * 客户端服务提供者
     */
    private static ClientServiceProvider SERVICE_PROVIDER;

    /**
     * 消息生产者工厂
     */
    private static RocketMQProducerFactory PRODUCER_FACTORY;

    /**
     * 获取配置属性
     *
     * @return 配置属性
     */
    public static RocketMQProperties getProperties() {
        if (PROPERTIES == null) {
            synchronized (RocketMQProperties.class) {
                if (PROPERTIES == null) {
                    PROPERTIES = ApplicationContextHolder.getBean(RocketMQProperties.class);
                }
            }
        }
        return PROPERTIES;
    }

    /**
     * 获取服务提供者
     *
     * @return 服务提供者实例
     */
    public static ClientServiceProvider getServiceProvider() {
        if (SERVICE_PROVIDER == null) {
            synchronized (ClientServiceProvider.class) {
                if (SERVICE_PROVIDER == null) {
                    SERVICE_PROVIDER = ApplicationContextHolder.getBean(ClientServiceProvider.class);
                }
            }
        }
        return SERVICE_PROVIDER;
    }

    /**
     * 获取消息工厂
     *
     * @return 消息工厂对象实例
     */
    public static MessageBuilder getMessageBuilder() {
        return getServiceProvider().newMessageBuilder();
    }

    /**
     * 获取消息生产者工厂
     *
     * @return 生产者工厂实例
     */
    public static RocketMQProducerFactory getProducerFactory() {
        if (PRODUCER_FACTORY == null) {
            synchronized (RocketMQProducerFactory.class) {
                if (PRODUCER_FACTORY == null) {
                    PRODUCER_FACTORY = ApplicationContextHolder.getBean(RocketMQProducerFactory.class);
                }
            }
        }
        return PRODUCER_FACTORY;
    }

    /**
     * 消息主题拆分
     *
     * @param original 原始消息主题
     * @return 消息主题/标签部分
     */
    public static Pair<String, String> topic(@NonNull String original) {
        int separator = original.indexOf(':');
        String topic = separator > 0 ? StringUtils.trim(original.substring(0, separator)) : original;
        String namespace = getProperties().getNamespace();
        if (StringUtils.nonEmpty(namespace)) {
            topic = namespace + topic;
        }
        String tag = separator > 0 ? StringUtils.trim(original.substring(separator + 1)) : StringUtils.EMPTY;
        return Pair.of(topic, tag);
    }

    /**
     * 消息序列化
     *
     * @param topic    消息主体
     * @param payload  消息内容
     * @param ordering 顺序消息标识
     * @param delaying 延时消息时间
     * @return 消息对象实例
     */
    public static Message serialize(@NonNull Pair<String, String> topic, @NonNull Object payload, String ordering,
                                    Long delaying) {
        byte[] body = JacksonUtils.serialize(payload).getBytes(StandardCharsets.UTF_8);
        String session = JacksonUtils.serialize(SessionContextHolder.getSession(false));
        MessageBuilder builder = getMessageBuilder().setTopic(topic.getLeft()).setBody(body)
                .addProperty(TYPE, payload.getClass().getName());
        if (Objects.nonNull(session)) {
            builder.addProperty(SESSION, session);
        }
        if (StringUtils.nonEmpty(topic.getRight())) {
            builder.setTag(topic.getRight());
        }
        if (StringUtils.nonEmpty(ordering)) {
            builder.setMessageGroup(ordering);
        }
        if (Objects.nonNull(delaying)) {
            long now = System.currentTimeMillis();
            long timestamp = now + Math.min(Math.max(delaying - now, 0), MAX_DELAY_TIME);
            builder.setDeliveryTimestamp(timestamp);
            builder.addProperty(DELAYING, String.valueOf(delaying));
        }
        return builder.build();
    }

    /**
     * 消息反序列化
     *
     * @param message 消息视图
     * @param type    消息类型
     */
    public static Object deserialize(@NonNull MessageView message, @NonNull Type type) {
        return JacksonUtils.deserialize(StandardCharsets.UTF_8.decode(message.getBody()).toString(), type);
    }

    /**
     * 消息反序列化
     *
     * @param message  消息视图
     * @param consumer 消息消费者
     */
    public static void deserialize(@NonNull MessageView message, @NonNull Consumer<Object> consumer) {
        String name = AssertUtils.nonempty(message.getProperties().get(TYPE), "Message type unspecified");
        Class<?> type;
        try {
            type = Class.forName(name);
        } catch (ClassNotFoundException e) {
            throw new RuntimeException(e);
        }
        deserialize(message, type, consumer);
    }

    /**
     * 消息反序列化
     *
     * @param message  消息视图
     * @param type     消息类型
     * @param consumer 消息消费者
     */
    public static void deserialize(@NonNull MessageView message, @NonNull Type type,
                                   @NonNull Consumer<Object> consumer) {
        Object payload = deserialize(message, type);
        Session session = JacksonUtils.deserialize(message.getProperties().get(SESSION), Session.class);
        SessionContextHolder.setSession(session);
        try {
            consumer.accept(payload);
        } finally {
            SessionContextHolder.clearSessionContext();
        }
    }

    /**
     * 获取消息延时时间
     *
     * @param message 消息视图
     * @return 延时时间戳
     */
    public static Long getMessageDelaying(MessageView message) {
        return StringUtils.ifEmpty(message.getProperties().get(DELAYING), Long::parseLong);
    }

    /**
     * 发送消息
     *
     * @param topic   消息主题
     * @param message 消息对象实例
     */
    public static void send(@NonNull String topic, @NonNull Message message) {
        Producer producer = getProducerFactory().getProducer(topic);
        try {
            producer.send(message);
        } catch (ClientException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 发送消息
     *
     * @param topic   消息主题
     * @param payload 消息内容
     */
    public static void send(@NonNull String topic, @NonNull Object payload) {
        send(topic, payload, null, (Long) null);
    }

    /**
     * 发送消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param ordering 顺序消息标识
     */
    public static void send(@NonNull String topic, @NonNull Object payload, String ordering) {
        send(topic, payload, ordering, (Long) null);
    }

    /**
     * 发送消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param delaying 延时消息时间
     */
    public static void send(@NonNull String topic, @NonNull Object payload, Duration delaying) {
        send(topic, payload, null, delaying == null ? null : System.currentTimeMillis() + delaying.toMillis());
    }

    /**
     * 发送消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param delaying 延时消息时间
     */
    public static void send(@NonNull String topic, @NonNull Object payload, Date delaying) {
        send(topic, payload, ObjectUtils.ifNull(delaying, Date::getTime));
    }

    /**
     * 发送消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param delaying 延时消息时间
     */
    public static void send(@NonNull String topic, @NonNull Object payload, Long delaying) {
        send(topic, payload, null, delaying);
    }

    /**
     * 发送消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param ordering 顺序消息标识
     * @param delaying 延时消息时间
     */
    public static void send(@NonNull String topic, @NonNull Object payload, String ordering, Duration delaying) {
        send(topic, payload, ordering, delaying == null ? null : System.currentTimeMillis() + delaying.toMillis());
    }

    /**
     * 发送消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param ordering 顺序消息标识
     * @param delaying 延时消息时间
     */
    public static void send(@NonNull String topic, @NonNull Object payload, String ordering, Date delaying) {
        send(topic, payload, ordering, ObjectUtils.ifNull(delaying, Date::getTime));
    }

    /**
     * 发送消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param ordering 顺序消息标识
     * @param delaying 延时消息时间
     */
    public static void send(@NonNull String topic, @NonNull Object payload, String ordering, Long delaying) {
        Pair<String, String> pair = topic(topic);
        send(pair.getLeft(), serialize(pair, payload, ordering, delaying));
    }

    /**
     * 发送异步消息
     *
     * @param topic   消息主题
     * @param message 消息对象实例
     */
    public static void sendAsync(@NonNull String topic, @NonNull Message message) {
        Producer producer = getProducerFactory().getProducer(topic);
        producer.sendAsync(message).exceptionally(e -> {
            log.error("RocketMQ message async send failed: {}, {}", topic, e.getMessage(), e);
            return null;
        });
    }

    /**
     * 发送异步消息
     *
     * @param topic   消息主题
     * @param payload 消息内容
     */
    public static void sendAsync(@NonNull String topic, @NonNull Object payload) {
        sendAsync(topic, payload, null, (Long) null);
    }

    /**
     * 发送异步消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param ordering 顺序消息标识
     */
    public static void sendAsync(@NonNull String topic, @NonNull Object payload, String ordering) {
        sendAsync(topic, payload, ordering, (Long) null);
    }

    /**
     * 发送异步消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param delaying 延时消息时间
     */
    public static void sendAsync(@NonNull String topic, @NonNull Object payload, Duration delaying) {
        sendAsync(topic, payload, delaying == null ? null : System.currentTimeMillis() + delaying.toMillis());
    }

    /**
     * 发送异步消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param delaying 延时消息时间
     */
    public static void sendAsync(@NonNull String topic, @NonNull Object payload, Date delaying) {
        sendAsync(topic, payload, ObjectUtils.ifNull(delaying, Date::getTime));
    }

    /**
     * 发送异步消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param delaying 延时消息时间
     */
    public static void sendAsync(@NonNull String topic, @NonNull Object payload, Long delaying) {
        sendAsync(topic, payload, null, delaying);
    }

    /**
     * 发送异步消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param ordering 顺序消息标识
     * @param delaying 延时消息时间
     */
    public static void sendAsync(@NonNull String topic, @NonNull Object payload, String ordering, Duration delaying) {
        sendAsync(topic, payload, ordering, delaying == null ? null : System.currentTimeMillis() + delaying.toMillis());
    }

    /**
     * 发送异步消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param ordering 顺序消息标识
     * @param delaying 延时消息时间
     */
    public static void sendAsync(@NonNull String topic, @NonNull Object payload, String ordering, Date delaying) {
        sendAsync(topic, payload, ordering, ObjectUtils.ifNull(delaying, Date::getTime));
    }

    /**
     * 发送异步消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param ordering 顺序消息标识
     * @param delaying 延时消息时间
     */
    public static void sendAsync(@NonNull String topic, @NonNull Object payload, String ordering, Long delaying) {
        Pair<String, String> pair = topic(topic);
        sendAsync(pair.getLeft(), serialize(pair, payload, ordering, delaying));
    }

    /**
     * 发送事务消息
     *
     * @param topic    消息主题
     * @param message  消息对象实例
     * @param executor 本地事务执行方法
     * @param checker  本地事务检测回调方法
     * @param <T>      消息类型
     */
    @SuppressWarnings("unchecked")
    public static <T> void sendTransaction(@NonNull String topic, @NonNull Message message,
                                           @NonNull Function<String, Boolean> executor,
                                           @NonNull Function<T, TransactionResolution> checker) {
        Producer producer = getProducerFactory().getProducer(topic, view -> {
            Reference<TransactionResolution> resolution = new Reference<>();
            deserialize(view, target -> resolution.setObject(checker.apply((T) target)));
            return Objects.requireNonNull(resolution.getObject(), "Transaction checker result must not be null");
        });
        try {
            Transaction transaction = producer.beginTransaction();
            MessageId id = producer.send(message, transaction).getMessageId();
            boolean submittable = Boolean.TRUE.equals(executor.apply(String.valueOf(id)));
            if (submittable) {
                transaction.commit();
            } else {
                transaction.rollback();
            }
        } catch (ClientException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 发送事务消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param executor 本地事务执行方法
     * @param checker  本地事务检测回调方法
     * @param <T>      消息类型
     */
    public static <T> void sendTransaction(@NonNull String topic, @NonNull T payload,
                                           @NonNull Function<String, Boolean> executor,
                                           @NonNull Function<T, TransactionResolution> checker) {
        sendTransaction(topic, payload, null, (Long) null, executor, checker);
    }

    /**
     * 发送事务消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param ordering 顺序消息标识
     * @param executor 本地事务执行方法
     * @param checker  本地事务检测回调方法
     * @param <T>      消息类型
     */
    public static <T> void sendTransaction(@NonNull String topic, @NonNull T payload, String ordering,
                                           @NonNull Function<String, Boolean> executor,
                                           @NonNull Function<T, TransactionResolution> checker) {
        sendTransaction(topic, payload, ordering, (Long) null, executor, checker);
    }

    /**
     * 发送事务消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param delaying 延时消息时间
     * @param executor 本地事务执行方法
     * @param checker  本地事务检测回调方法
     * @param <T>      消息类型
     */
    public static <T> void sendTransaction(@NonNull String topic, @NonNull T payload, Duration delaying,
                                           @NonNull Function<String, Boolean> executor,
                                           @NonNull Function<T, TransactionResolution> checker) {
        Long timestamp = delaying == null ? null : System.currentTimeMillis() + delaying.toMillis();
        sendTransaction(topic, payload, timestamp, executor, checker);
    }

    /**
     * 发送事务消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param delaying 延时消息时间
     * @param executor 本地事务执行方法
     * @param checker  本地事务检测回调方法
     * @param <T>      消息类型
     */
    public static <T> void sendTransaction(@NonNull String topic, @NonNull T payload, Date delaying,
                                           @NonNull Function<String, Boolean> executor,
                                           @NonNull Function<T, TransactionResolution> checker) {
        sendTransaction(topic, payload, ObjectUtils.ifNull(delaying, Date::getTime), executor, checker);
    }

    /**
     * 发送事务消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param delaying 延时消息时间
     * @param executor 本地事务执行方法
     * @param checker  本地事务检测回调方法
     * @param <T>      消息类型
     */
    public static <T> void sendTransaction(@NonNull String topic, @NonNull T payload, Long delaying,
                                           @NonNull Function<String, Boolean> executor,
                                           @NonNull Function<T, TransactionResolution> checker) {
        sendTransaction(topic, payload, null, delaying, executor, checker);
    }

    /**
     * 发送事务消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param ordering 顺序消息标识
     * @param delaying 延时消息时间
     * @param executor 本地事务执行方法
     * @param checker  本地事务检测回调方法
     * @param <T>      消息类型
     */
    public static <T> void sendTransaction(@NonNull String topic, @NonNull T payload, String ordering,
                                           Duration delaying, @NonNull Function<String, Boolean> executor,
                                           @NonNull Function<T, TransactionResolution> checker) {
        Long timestamp = delaying == null ? null : System.currentTimeMillis() + delaying.toMillis();
        sendTransaction(topic, payload, ordering, timestamp, executor, checker);
    }

    /**
     * 发送事务消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param ordering 顺序消息标识
     * @param delaying 延时消息时间
     * @param executor 本地事务执行方法
     * @param checker  本地事务检测回调方法
     * @param <T>      消息类型
     */
    public static <T> void sendTransaction(@NonNull String topic, @NonNull T payload, String ordering,
                                           Date delaying, @NonNull Function<String, Boolean> executor,
                                           @NonNull Function<T, TransactionResolution> checker) {
        sendTransaction(topic, payload, ordering, ObjectUtils.ifNull(delaying, Date::getTime), executor, checker);
    }

    /**
     * 发送事务消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param ordering 顺序消息标识
     * @param delaying 延时消息时间
     * @param executor 本地事务执行方法
     * @param checker  本地事务检测回调方法
     * @param <T>      消息类型
     */
    public static <T> void sendTransaction(@NonNull String topic, @NonNull T payload, String ordering,
                                           Long delaying, @NonNull Function<String, Boolean> executor,
                                           @NonNull Function<T, TransactionResolution> checker) {
        Pair<String, String> pair = topic(topic);
        sendTransaction(pair.getLeft(), serialize(pair, payload, ordering, delaying), executor, checker);
    }
}
