package com.chelvc.framework.security.context;

import java.util.Objects;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.util.AESUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.security.config.SecurityProperties;
import lombok.extern.slf4j.Slf4j;

/**
 * 安全处理上下文工具类
 * <p>
 * 角色资源授权信息从Nacos配置中心获取并动态更新
 *
 * @author Woody
 * @date 2023/1/1
 */
@Slf4j
public class SecurityContextHolder {
    /**
     * 加解密字符串特征前缀
     */
    public static final String CODEC_CHARACTERISTIC_PREFIX = "ENC(";

    /**
     * 加解密字符串特征后缀
     */
    public static final String CODEC_CHARACTERISTIC_SUFFIX = ")";

    /**
     * 配置属性
     */
    private static SecurityProperties PROPERTIES;

    /**
     * 获取配置属性
     *
     * @return 配置属性
     */
    public static SecurityProperties getProperties() {
        if (PROPERTIES == null) {
            synchronized (SecurityProperties.class) {
                PROPERTIES = ApplicationContextHolder.getBean(SecurityProperties.class);
            }
        }
        return PROPERTIES;
    }

    /**
     * 判断是否启用安全校验
     *
     * @return true/false
     */
    public static boolean isEnabled() {
        return getProperties().isEnabled();
    }

    /**
     * 判断是否忽略安全校验失败请求
     *
     * @return true/false
     */
    public static boolean isFailureIgnored() {
        return ApplicationContextHolder.getSafeProperty("security.failure.ignored", boolean.class, false);
    }

    /**
     * 数据加密
     *
     * @param plaintext 数据明文
     * @return 数据密文
     */
    public static String encrypt(String plaintext) {
        if (StringUtils.isEmpty(plaintext) || !isEnabled()) {
            return plaintext;
        }
        SecurityProperties properties = getProperties();
        String secret = Objects.requireNonNull(properties.getSecret(), "Invalid secret");
        String device = Objects.requireNonNull(SessionContextHolder.getDevice(), "Invalid device");
        String iv = StringUtils.substring(device, 0, 16);
        String ciphertext = AESUtils.encode(plaintext, secret, iv);
        if (log.isDebugEnabled()) {
            log.debug("Data encrypt: {} -> {}", plaintext, ciphertext);
        }
        return CODEC_CHARACTERISTIC_PREFIX + ciphertext + CODEC_CHARACTERISTIC_SUFFIX;
    }

    /**
     * 数据加密
     *
     * @param plaintext 数据明文
     * @param safely    是否使用安全模式（如果加密失败则返回明文）
     * @return 数据密文
     */
    public static String encrypt(String plaintext, boolean safely) {
        // 非安全模式
        if (!safely) {
            return encrypt(plaintext);
        }

        // 安全模式
        try {
            return encrypt(plaintext);
        } catch (Exception e) {
            // 如果加密失败则返回明文
            log.error("Data encrypt failed: {}", e.getMessage());
            return plaintext;
        }
    }

    /**
     * 数据解密
     *
     * @param ciphertext 数据密文
     * @return 数据明文
     */
    public static String decrypt(String ciphertext) {
        if (StringUtils.isEmpty(ciphertext) || !(ciphertext.startsWith(CODEC_CHARACTERISTIC_PREFIX)
                && ciphertext.endsWith(CODEC_CHARACTERISTIC_SUFFIX))) {
            return ciphertext;
        }
        SecurityProperties configuration = getProperties();
        String secret = Objects.requireNonNull(configuration.getSecret(), "Invalid secret");
        String device = Objects.requireNonNull(SessionContextHolder.getDevice(), "Invalid device");
        String iv = StringUtils.substring(device, 0, 16);
        ciphertext = ciphertext.substring(
                CODEC_CHARACTERISTIC_PREFIX.length(), ciphertext.length() - CODEC_CHARACTERISTIC_SUFFIX.length()
        );
        String plaintext = AESUtils.decode(ciphertext, secret, iv);
        if (log.isDebugEnabled()) {
            log.debug("Data decrypt: {} -> {}", ciphertext, plaintext);
        }
        return plaintext;
    }
}
