package com.chelvc.framework.security.interceptor;

import java.io.IOException;

import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.security.annotation.Encrypt;
import com.chelvc.framework.security.context.SecurityContextHolder;
import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.BeanProperty;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.ser.ContextualSerializer;
import com.fasterxml.jackson.databind.ser.std.StdSerializer;

/**
 * 敏感数据加密序列化处理器
 *
 * @author Woody
 * @date 2023/1/1
 */
public class JacksonEncryptSerializer extends StdSerializer<String> implements ContextualSerializer {
    private boolean enabled = true;

    public JacksonEncryptSerializer() {
        super(String.class);
    }

    /**
     * 判断是否受版本控制
     *
     * @param annotation 加密注解实例
     * @return true/false
     */
    private boolean isVersioning(Encrypt annotation) {
        return ObjectUtils.isEmpty(annotation.versioning()) || SessionContextHolder.isVersion(annotation.versioning());
    }

    /**
     * 判断是否是可加密属性
     *
     * @param property 属性对象
     * @return true/false
     */
    private boolean isEncryptProperty(BeanProperty property) {
        return property.getType().getRawClass() == String.class;
    }

    /**
     * 获取属性加密注解实例
     *
     * @param property 属性对象
     * @return 加密注解实例
     */
    private Encrypt getEncryptAnnotation(BeanProperty property) {
        return property.getAnnotation(Encrypt.class);
    }

    @Override
    public void serialize(String plaintext, JsonGenerator generator, SerializerProvider provider) throws IOException {
        if (!this.enabled || StringUtils.isEmpty(plaintext)) {
            generator.writeString(plaintext);
        } else {
            generator.writeString(SecurityContextHolder.encrypt(plaintext, true));
        }
    }

    @Override
    public JsonSerializer<?> createContextual(SerializerProvider provider, BeanProperty property) {
        if (property == null) {
            return provider.getDefaultNullValueSerializer();
        }
        if (!this.isEncryptProperty(property)) {
            this.enabled = false;
        } else {
            Encrypt annotation = this.getEncryptAnnotation(property);
            this.enabled = annotation == null || this.isVersioning(annotation);
        }
        return this;
    }
}
