package com.chelvc.framework.security.interceptor;

import java.io.IOException;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.base.model.Result;
import com.chelvc.framework.base.util.HttpUtils;
import com.chelvc.framework.base.util.SpringUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.security.config.SecurityProperties;
import com.chelvc.framework.security.context.SecurityContextHolder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.annotation.Order;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;

/**
 * 请求安全拦截器
 *
 * @author Woody
 * @date 2023/1/1
 */
@Slf4j
@Order(10)
@Component
public class RequestSecurityInterceptor implements Filter {
    /**
     * 安全性校验
     *
     * @param request  Http请求对象
     * @param response Http相应对象
     * @return true/false
     * @throws IOException I/O操作异常
     */
    private boolean validate(HttpServletRequest request, HttpServletResponse response) throws IOException {
        // 判断开关是否开启
        if (!SecurityContextHolder.isEnabled()) {
            return true;
        }

        // 判断是否需要放行
        String uri = HttpUtils.getRequestURI(request);
        SecurityProperties.Request configuration = SecurityContextHolder.getProperties().getRequest();
        if (SpringUtils.isPath(uri, configuration.getPermit()) || (ObjectUtils.nonEmpty(configuration.getRequire())
                && !SpringUtils.isPath(uri, configuration.getRequire()))) {
            return true;
        }

        // 判断请求是否合法
        boolean deadline = true;
        long duration = Math.max(configuration.getDuration(), 1000);
        String device = request.getHeader(SessionContextHolder.HEADER_DEVICE);
        String platform = request.getHeader(SessionContextHolder.HEADER_PLATFORM);
        String terminal = request.getHeader(SessionContextHolder.HEADER_TERMINAL);
        String version = request.getHeader(SessionContextHolder.HEADER_VERSION);
        String timestamp = request.getHeader(SessionContextHolder.HEADER_TIMESTAMP);
        if (StringUtils.nonEmpty(device) && StringUtils.nonEmpty(platform) && StringUtils.nonEmpty(terminal)
                && StringUtils.nonEmpty(version) && StringUtils.nonEmpty(timestamp) && StringUtils.isNumber(timestamp)
                && (deadline = System.currentTimeMillis() - Long.parseLong(timestamp) <= duration)) {
            return true;
        }

        // 无效请求
        log.warn(SessionContextHolder.getLoggingMessage(request, HttpStatus.FORBIDDEN, "Invalid request"));
        if (SecurityContextHolder.isFailureIgnored()) {
            return true;
        }
        if (deadline) {
            response.setStatus(HttpStatus.FORBIDDEN.value());
            SessionContextHolder.response(response, Result.build(HttpStatus.FORBIDDEN));
        } else {
            response.setStatus(HttpStatus.NOT_ACCEPTABLE.value());
            Result<?> result = Result.build(HttpStatus.NOT_ACCEPTABLE, null, "请校准系统时间");
            SessionContextHolder.response(response, result);
        }
        return false;
    }

    @Override
    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain)
            throws IOException, ServletException {
        if (this.validate((HttpServletRequest) request, (HttpServletResponse) response)) {
            chain.doFilter(new RepeatableRequestWrapper((HttpServletRequest) request), response);
        }
    }
}
