package com.chelvc.framework.sms.support;

import java.util.Map;
import java.util.Objects;

import com.chelvc.framework.common.util.CodecUtils;
import com.chelvc.framework.sms.NormalSmsHandler;
import com.chelvc.framework.sms.config.TengyuSmsProperties;
import com.google.common.collect.ImmutableMap;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

/**
 * 腾域营销短信操作实现
 *
 * @author Woody
 * @date 2023/1/1
 */
@Slf4j
@Component
@ConditionalOnBean(TengyuSmsProperties.class)
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class TengyuSmsHandler implements NormalSmsHandler {
    /**
     * 发送短信接口地址
     */
    private static final String SEND_SMS_URL = "https://send.it1688.com.cn:8443/sms/api/sendMessage";

    private final RestTemplate restTemplate;
    private final TengyuSmsProperties properties;

    @Override
    public boolean send(@NonNull String content, @NonNull String... mobiles) {
        if (mobiles.length == 0) {
            return false;
        }
        long timestamp = System.currentTimeMillis();
        String username = this.properties.getUsername();
        String password = this.properties.getPassword();
        String sign = CodecUtils.md5(username + content + timestamp + CodecUtils.md5(password));
        Map<String, Object> body = ImmutableMap.of(
                "userName", username,
                "content", content,
                "phoneList", mobiles,
                "timestamp", timestamp,
                "sign", sign
        );
        boolean debug = log.isDebugEnabled();
        if (debug) {
            log.debug("Sms request: {}, {}", SEND_SMS_URL, body);
        }
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        HttpEntity<?> entity = new HttpEntity<>(body, headers);
        Map<?, ?> response;
        try {
            response = this.restTemplate.exchange(SEND_SMS_URL, HttpMethod.POST, entity, Map.class).getBody();
        } catch (Exception e) {
            log.warn("Tengyu sms send failed: {}", e.getMessage());
            return false;
        }
        if (debug) {
            log.debug("Tengyu sms response: {}", response);
        }
        if (response == null || !Objects.equals(response.get("code"), 0)) {
            log.warn("Tengyu sms send failed: {}", response);
            return false;
        }
        return true;
    }
}
